@@ -21,6 +21,26 @@
 
 #include <arrow/c/bridge.h>
 
+// [[arrow::export]]
+arrow::r::Pointer<struct ArrowSchema> allocate_arrow_schema() { return {}; }
+
+// [[arrow::export]]
+void delete_arrow_schema(arrow::r::Pointer<struct ArrowSchema> ptr) { ptr.finalize(); }
+
+// [[arrow::export]]
+arrow::r::Pointer<struct ArrowArray> allocate_arrow_array() { return {}; }
+
+// [[arrow::export]]
+void delete_arrow_array(arrow::r::Pointer<struct ArrowArray> ptr) { ptr.finalize(); }
+
+// [[arrow::export]]
+arrow::r::Pointer<struct ArrowArrayStream> allocate_arrow_array_stream() { return {}; }
+
+// [[arrow::export]]
+void delete_arrow_array_stream(arrow::r::Pointer<struct ArrowArrayStream> ptr) {
+  ptr.finalize();
+}
+
 // [[arrow::export]]
 std::shared_ptr<arrow::Array> ImportArray(arrow::r::Pointer<struct ArrowArray> array,
                                           arrow::r::Pointer<struct ArrowSchema> schema) {
@@ -41,16 +61,20 @@ std::shared_ptr<arrow::Schema> ImportSchema(
 }
 
 // [[arrow::export]]
-arrow::r::Pointer<struct ArrowSchema> allocate_arrow_schema() { return {}; }
+std::shared_ptr<arrow::Field> ImportField(arrow::r::Pointer<struct ArrowSchema> field) {
+  return ValueOrStop(arrow::ImportField(field));
+}
 
 // [[arrow::export]]
-void delete_arrow_schema(arrow::r::Pointer<struct ArrowSchema> ptr) { ptr.finalize(); }
+std::shared_ptr<arrow::DataType> ImportType(arrow::r::Pointer<struct ArrowSchema> type) {
+  return ValueOrStop(arrow::ImportType(type));
+}
 
 // [[arrow::export]]
-arrow::r::Pointer<struct ArrowArray> allocate_arrow_array() { return {}; }
-
-// [[arrow::export]]
-void delete_arrow_array(arrow::r::Pointer<struct ArrowArray> ptr) { ptr.finalize(); }
+std::shared_ptr<arrow::RecordBatchReader> ImportRecordBatchReader(
+    arrow::r::Pointer<struct ArrowArrayStream> stream) {
+  return ValueOrStop(arrow::ImportRecordBatchReader(stream));
+}
 
 // [[arrow::export]]
 void ExportType(const std::shared_ptr<arrow::DataType>& type,
@@ -58,6 +82,12 @@ void ExportType(const std::shared_ptr<arrow::DataType>& type,
   StopIfNotOk(arrow::ExportType(*type, ptr));
 }
 
+// [[arrow::export]]
+void ExportField(const std::shared_ptr<arrow::Field>& field,
+                 arrow::r::Pointer<struct ArrowSchema> ptr) {
+  StopIfNotOk(arrow::ExportField(*field, ptr));
+}
+
 // [[arrow::export]]
 void ExportSchema(const std::shared_ptr<arrow::Schema>& schema,
                   arrow::r::Pointer<struct ArrowSchema> ptr) {
@@ -78,4 +108,10 @@ void ExportRecordBatch(const std::shared_ptr<arrow::RecordBatch>& batch,
   StopIfNotOk(arrow::ExportRecordBatch(*batch, array_ptr, schema_ptr));
 }
 
+// [[arrow::export]]
+void ExportRecordBatchReader(const std::shared_ptr<arrow::RecordBatchReader>& reader,
+                             arrow::r::Pointer<struct ArrowArrayStream> stream_ptr) {
+  StopIfNotOk(arrow::ExportRecordBatchReader(reader, stream_ptr));
+}
+
 #endif
